/* MinuteMonitor, log duration and display usage for the N900
 * Copyright (c) 2012 David Fries <David@Fries.net> http://david.fries.net/
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "DBLog.h"
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <sstream>
#include <sqlite3.h>

using namespace std;

int DBLog::Verbose;

DBLog::DBLog() :
	Db(NULL)
{
}

DBLog::~DBLog()
{
	sqlite3_close(Db);
}

bool DBLog::Open()
{
	const char *home=getenv("HOME");
	if(!home)
		return false;

	string file=string(home)+"/.rtcom-eventlogger/el-v1.db";
	if(sqlite3_open(file.c_str(), &Db))
	{
		cerr << "Error opening " << file << ' ' << sqlite3_errmsg(Db)
			<< endl;
		return false;
	}

	return true;
}

void DBLog::CallSetup(const char *path, const char *stream,
	const char * /*media*/, uint32_t /*ui1*/, uint32_t /*ui2*/, bool /*b*/)
{
	LastSetupStream=stream;
	Call &call=StreamId[LastSetupStream];
	call.id=-1;
	call.Db=Db;
	gettimeofday(&call.start_time, NULL);
	call.storage_time.tv_sec=0;
	call.storage_time.tv_usec=0;
	if(Verbose)
	{
		cout << __func__ << ' ' << path << ' ' << stream << endl;
		Status();
	}
}

void DBLog::CallLog(int id, const char *protocol, const char * /*contact*/,
	const char * /*str3*/, const char * /*str4*/, const char * /*type*/)
{
	if(LastSetupStream.empty())
	{
		cerr << "LastSetupStream is empty\n";
		return;
	}
	StreamIdType::iterator iter=StreamId.find(LastSetupStream);
	if(iter==StreamId.end())
	{
		cerr << "Stream entry not found\n";
		return;
	}
	iter->second.id=id;
	gettimeofday(&iter->second.storage_time, NULL);
	if(Verbose)
	{
		cout << __func__ << ' ' << id << ' ' << protocol
			<< " assuming\n" << LastSetupStream << endl;
		Status();
	}
}

void DBLog::CallEnded(const char *stream, int unknown)
{
	// rows updated
	StreamIdType::iterator iter=StreamId.find(stream);
	if(iter==StreamId.end())
	{
		cerr << "Stream entry not found\n";
		return;
	}
	gettimeofday(&iter->second.end_time, NULL);

	if(Verbose)
	{
		cout << __func__ << ' ' << stream << ' ' << unknown
			<< " id " << iter->second.id << endl;
		Status();
	}

	stringstream ss;
	ss << "select id, storage_time, start_time, end_time, local_uid, "
		"local_name, remote_uid from Events where id='"
		<< iter->second.id << "';\n";
	string query=ss.str();
	char *errmsg=NULL;
	sqlite3_exec(Db, query.c_str(), Callback, &iter, &errmsg);
	if(errmsg)
	{
		cerr << "Error executing " << query << errmsg << endl;
		sqlite3_free(errmsg);
	}
	LastSetupStream.clear();
	StreamId.erase(iter);
}

int DBLog::Callback(void *userdata, int columns, char **value,
	char **header)
{
	if(Verbose)
	{
		for(int i=0; i<columns; ++i)
		{
			cout << header[i] << ": "
				<< (value[i] ? value[i] : "<NULL>") << endl;
		}
	}
	// id, storage_time, start_time, end_time
	if(columns < 4)
	{
		cerr << "Too few columns\n";
		exit(1);
	}
	StreamIdType::iterator &iter=*(StreamIdType::iterator*)userdata;
	Call &call=(*(StreamIdType::iterator*)userdata)->second;
	if(call.id != atoi(value[0]))
	{
		cerr << "id didn't match\n";
		return 1;
	}
	if(Verbose)
	{
		cout << "recorded storage " << call.storage_time.tv_sec
			<< " start " << call.start_time.tv_sec << endl;
	}
	if(abs(call.storage_time.tv_sec - atoi(value[1])) > 30 ||
		abs(call.start_time.tv_sec - atoi(value[2])) > 30)
	{
		cerr << "logged time differs too much\n";
		return 1;
	}
	// only override the value if it's zero
	if(strcmp(value[3], "0"))
	{
		cerr << "end_time is " << value[3] << endl;
		return 1;
	}
	stringstream ss;
	ss << "update Events set end_time='" << call.end_time.tv_sec
		<< "' where id='" << call.id << "';";
	string query=ss.str();
	char *errmsg=NULL;
	sqlite3_exec(call.Db, query.c_str(), NULL, &iter, &errmsg);
	if(errmsg)
	{
		cerr << "Error executing " << query << errmsg << endl;
		sqlite3_free(errmsg);
	}
	int rows=sqlite3_changes(call.Db);
	if(rows != 1)
	{
		cerr << "Error " << rows << " were modified\n";
		return 1;
	}
	return 0;
}

void DBLog::Status()
{
	cout << "LastSetupStream " << LastSetupStream << endl;
	for(StreamIdType::iterator iter=StreamId.begin(); iter!=StreamId.end();
		++iter)
	{
		Call &entry=iter->second;
		cout << iter->first << ' ' << entry.id << ' '
			<< entry.start_time.tv_sec << endl;
	}
}
