/* MinuteMonitor, log duration and display usage for the N900
 * Copyright (c) 2012 David Fries <David@Fries.net> http://david.fries.net/
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _QML_STORE_H
#define _QML_STORE_H 1

#include <QObject>
#include <QSharedData>
#include <QString>
#include <QVariant>
#include <QAbstractListModel>
#include <QtDeclarative>
#include "Store.h"

class CallType : public QObject
{
	Q_OBJECT
	Q_ENUMS(CallTypeEnum)
	Q_FLAGS(CallRateEnum)
	Q_ENUMS(CallDirection)
public:
	// 0 uses minutes, 1 free call, 2 VoIP
	// This is the classification of that contact not for a given
	// call entry if that was in the evening and free, but
	// otherwise would cost.
	enum CallTypeEnum {CALL_COST, CALL_FREE, CALL_VOIP, CALL_UNKNOWN};
	// bit field, indicate which types of calls were received (for
	// contact list which could include both)
	enum CallRateEnum
	{
		UNKNOWN_RATE=DBMonitor::UNKNOWN_RATE,
		DAYTIME_RATE=DBMonitor::DAYTIME_RATE,
		EVENING_RATE=DBMonitor::EVENING_RATE,
		// will no longer combine different rates
		BOTH_RATE=DBMonitor::BOTH_RATE,
	};
	enum CallDirection
	{
		CALL_NEITHER=DBMonitor::CALL_NEITHER,
		// bitfield for call direction
		CALL_IN=DBMonitor::CALL_IN,
		CALL_OUT=DBMonitor::CALL_OUT,
		// an individual call is in or out, but for the contact list
		// might have calls both in and out for a given contact
		CALL_BOTH=DBMonitor::CALL_BOTH
	};
};

struct LabelData
{
	LabelData() : type(CallType::CALL_UNKNOWN),
		direction(CallType::CALL_NEITHER) {}
	LabelData(const QString &start, const QString &name, const QString &min,
		const QString &uid, CallType::CallTypeEnum type_arg,
		CallType::CallRateEnum start_rate,
		CallType::CallDirection direction_arg) :
		start_time(start), remote_name(name), minutes(min),
		remote_uid(uid), type(type_arg), rate(start_rate),
		direction(direction_arg) {}
	QString start_time, remote_name, minutes, remote_uid;
	CallType::CallTypeEnum type;
	CallType::CallRateEnum rate;
	CallType::CallDirection direction;
};
QML_DECLARE_TYPE(LabelData)

class LabelModel : public QAbstractListModel
{
	Q_OBJECT
public:
	LabelModel(QObject *parent=NULL);
	enum ItemRoles
	{
		start_time=Qt::UserRole+1,
		remote_name,
		minutes,
		remote_uid,
		type,
		rate,
		direction,
	};

	int rowCount(const QModelIndex &/*parent=QModelIndex()*/) const
	{ return Data.size(); }
	QVariant data(const QModelIndex &index, int role = Qt::DisplayRole)
		const;
	void Resize(int count);
	const LabelData& At(int row) const { return Data[row]; }
	void Set(int row, const LabelData &data);
	// Update any label for the remote_uid to type.
	void UpdateLabels(const QString &remote_uid,
		CallType::CallTypeEnum type);
private:
	std::vector<LabelData> Data;
};

// Export and interface QML with Store
class QmlStore : public QObject, public Store
{
	Q_OBJECT
	Q_PROPERTY(QString statsHeader READ StatsHeader NOTIFY Updated)
	Q_PROPERTY(QString stats_1 READ Stats_1 NOTIFY Updated)
	Q_PROPERTY(QString stats_2 READ Stats_2 NOTIFY Updated)
	Q_PROPERTY(QString stats_3 READ Stats_3 NOTIFY Updated)
	Q_PROPERTY(QString statsFooter READ StatsFooter NOTIFY Updated)
	Q_PROPERTY(int minutesAvailable READ GetMinutesAvailable
		WRITE SetMinutesAvailable NOTIFY Updated)
	Q_PROPERTY(int periodEnd READ GetPeriodEnd
		WRITE SetPeriodEnd NOTIFY Updated)
	Q_PROPERTY(QString eveningEnd READ GetEveningEnd
		WRITE SetEveningEnd NOTIFY Updated)
	Q_PROPERTY(QString eveningBegin READ GetEveningBegin
		WRITE SetEveningBegin NOTIFY Updated)
	Q_PROPERTY(bool incomingFree READ GetIncomingFree
		WRITE SetIncomingFree NOTIFY Updated)
	Q_PROPERTY(bool nonprimeFree READ GetNonprimeFree
		WRITE SetNonprimeFree NOTIFY Updated)
signals:
	// emitted each time data is loaded from the database
	void Updated();
public:
	// a header, three columns of data, and footer
	QString StatsHeader() const;
	QString Stats_1();
	QString Stats_2() const { return StatStr2; }
	QString Stats_3() const { return StatStr3; }
	QString StatsFooter() const;

	// Call one to setup the data in the list.

	// one entry per call in the current period
	Q_INVOKABLE void CallLog();
	// one entry per contact in the current period
	Q_INVOKABLE void ContactLog();
	// load telephone contacts in the database
	Q_INVOKABLE void LoadTelContacts();

	// label information for use as a call log
	LabelModel* Labels() { return &Model; }

	// previous and next period that is
	Q_INVOKABLE bool LoadPrevious();
	Q_INVOKABLE bool LoadNext();

	void SetMinutesAvailable(int min) { DB.SetMinutesAvailable(min); }
	int GetMinutesAvailable() const { return DB.GetMinutesAvailable(); }
	void SetPeriodEnd(int day_of_month) { DB.SetPeriodEnd(day_of_month); }
	int GetPeriodEnd() const { return DB.GetPeriodEnd(); }
	void SetEveningEnd(const QString &t);
	QString GetEveningEnd() const;
	void SetEveningBegin(const QString &t);
	QString GetEveningBegin() const;
	void SetIncomingFree(bool free) { DB.SetIncomingFree(free); }
	bool GetIncomingFree() const { return DB.GetIncomingFree(); }
	void SetNonprimeFree(bool free) { DB.SetNonprimeFree(free); }
	bool GetNonprimeFree() const { return DB.GetNonprimeFree(); }
	// Write the current configuration values to the config database.
	Q_INVOKABLE bool WriteConfig();

	Q_INVOKABLE void AddFreeContact(const QString &remote_uid);
	Q_INVOKABLE void RemoveFreeContact(const QString &remote_uid);

private:
	LabelModel Model;
	// Generate the three strings at the same time to keep all the logic
	// together.
	QString StatStr1, StatStr2, StatStr3;

	// Fill the LabelList with the given log data.
	void FillLabelList(const DBMonitor::LogType &log);
	CallType::CallTypeEnum GetCallType(
		const DBMonitor::ContactsHashType &free_call_set,
		const std::string &remote_uid);
	static QString SecToTime(int sec);
	static int TimeToSec(const QString &str);
};

#endif // _QML_STORE_H
