/* MinuteMonitor, log duration and display usage for the N900
 * Copyright (c) 2012 David Fries <David@Fries.net> http://david.fries.net/
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Store.h"
#include <stdio.h>

using namespace std;

Store::Store() :
	PStart(0),
	PEnd(0),
	Anytime(0),
	Incoming(0),
	Evening(0),
	Free(0),
	VoIP(0)
{
}

bool Store::Open()
{
	bool result=DB.Open();
	SetPeriod(time(NULL));
	return result;
}

time_t Store::SetPeriod(const time_t in_period)
{
	struct tm *t=localtime(&in_period);
	// if it ended before the current day +month
	if(DB.GetPeriodEnd() < t->tm_mday)
		++t->tm_mon;
	// end of the day
	t->tm_sec=59;
	t->tm_min=59;
	t->tm_hour=23;
	t->tm_mday=DB.GetPeriodEnd();
	t->tm_yday=-1;
	// avoid daylight savings time crossing adjustments, call it local time
	t->tm_isdst=-1;
	// the month the period ends
	int end_month=t->tm_mon;
	// find out if the period end is after the last day of this month
	PEnd=mktime(t);
	if(t->tm_mon > end_month)
	{
		// want the last day of the previous month (1 is the 1st)
		t->tm_mday=0;
		PEnd=mktime(t);
	}

	// start of the period, the following day only the previous month
	t->tm_mday=DB.GetPeriodEnd()+1;
	--t->tm_mon;
	t->tm_sec=0;
	t->tm_min=0;
	t->tm_hour=0;
	t->tm_isdst=-1; // call it local time whatever the end time is
	PStart=mktime(t);
	if(t->tm_mon==end_month)
	{
		// the month advanced, reset the day to the start of the month
		t->tm_mday=1;
		PStart=mktime(t);
	}
	return PStart;
}

bool Store::Load()
{
	if(!DB.Load(PStart, PEnd))
		return false;
	DBMonitor::LogType &log=DB.DataSetEdit();
	int duration;
	struct tm *t;
	int sec;
	Anytime=Incoming=Evening=Free=VoIP=0;
	bool incoming_free=DB.GetIncomingFree();
	bool nonprime_free=DB.GetNonprimeFree();
	for(size_t i=0; i<log.size(); ++i)
	{
		DBMonitor::LogEntry &entry=log[i];
		duration=entry.ceil_min;
		if(duration <= 0)
			continue;
		if(entry.VoIP)
		{
			VoIP+=duration;
			continue;
		}
		if(DB.IsFreeContact(entry.remote_uid))
		{
			//printf("Free    %d + %d\n", Free, duration);
			Free+=duration;
			continue;
		}
		if(incoming_free && entry.direction == DBMonitor::CALL_IN)
		{
			//printf("In      %d + %d\n", Incoming, duration);
			Incoming+=duration;
			continue;
		}
		if(nonprime_free)
		{
			time_t when=entry.start_time;
			t=localtime(&when);
			// weekend
			if(t->tm_wday == 0 || t->tm_wday == 6)
			{
				/*
				printf("Weekend %d + %d, day %d\n", Evening,
					duration, t->tm_wday);
					*/
				Evening+=duration;
				entry.rate=DBMonitor::EVENING_RATE;
				continue;
			}
			sec = t->tm_sec + t->tm_min*60 + t->tm_hour*60*60;
			if(sec <= DB.GetEveningEnd() ||
				DB.GetEveningBegin() <= sec)
			{
				/*
				printf("Evening %d + %d, %d:%02d:%02d, "
					"%d to %d\n",
					Evening, duration, t->tm_hour,
					t->tm_min, t->tm_sec,
					DB.GetEveningEnd(),
					DB.GetEveningBegin());
					*/
				Evening+=duration;
				entry.rate=DBMonitor::EVENING_RATE;
				continue;
			}
		}
		//printf("Anytime %d + %d\n", Anytime, duration);
		Anytime+=duration;
		entry.rate=DBMonitor::DAYTIME_RATE;
	}
	return true;
}

bool Store::LoadPrevious()
{
	SetPeriod(PStart - 2*7*24*60*60);
	return Load();
}

bool Store::LoadNext()
{
	SetPeriod(PEnd + 2*7*24*60*60);
	return Load();
}

std::string Store::DateToString(time_t when) const
{
	tm *t=localtime(&when);
	char buf[80];
	strftime(buf, sizeof(buf), "%m-%d-%Y", t);
	return buf;
}

std::string Store::TimestampToString(time_t when) const
{
	tm *t=localtime(&when);
	char buf[80];
	strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M", t);
	return buf;
}

std::string Store::MinutesToString(int sec)
{
	char buf[80];
	bool neg=false;
	if(sec<0)
	{
		neg=true;
		sec*=-1;
	}
	
	sprintf(buf, "%02d:%02d:%02d", sec/3600, sec/60%60, sec%60);
	if(neg)
		return string("-")+buf;
	return buf;
}
